#!/usr/bin/env python

"""
XSL-based form templating.
"""

import Constants
import libxslt

"""
import libxml2

def quiet(context, s):
    pass

libxml2.registerErrorHandler(quiet, None)
"""

def path_to_node(node, attribute_ref, name, multivalue=0):

    """
    Generate an XSLForms path to the given 'node', producing an attribute
    reference if 'attribute_ref' is true; for example:

    /package#1/discriminators#5/discriminator#1/category

    Otherwise an element reference is produced; for example:

    /package#1/discriminators#5/discriminator#1

    Use the given 'name' to complete the path if an attribute reference is
    required (and if a genuine attribute is found at the context node -
    otherwise 'name' will be None and the context node will be treated like an
    attribute).

    If 'multivalue' is true, produce an attribute reference using the given
    'name' of the following form:

    /package#1/categories#1/category##name
    """

    l = []
    # Skip attribute reference.
    if node.type == "attribute":
        node = node.parent
    # Manually insert the attribute name if defined.
    if attribute_ref:
        # A real attribute is referenced.
        if name is not None:
            l.insert(0, name)
            if multivalue:
                l.insert(0, Constants.pair_separator)
                l.insert(0, Constants.pair_separator)
                l.insert(0, node.name)
                node = node.parent
            l.insert(0, Constants.path_separator)
        # Otherwise, treat the element name as an attribute name.
        else:
            l.insert(0, node.name)
            l.insert(0, Constants.path_separator)
            node = node.parent
    # Element references.
    while node is not None and node.type != "document_xml":
        l.insert(0, str(int(node.xpathEval("count(preceding-sibling::*) + 1"))))
        l.insert(0, Constants.pair_separator)
        l.insert(0, node.name)
        l.insert(0, Constants.path_separator)
        node = node.parent
    return "".join(l)

def path_to_context(context, attribute_ref, multivalue_name=None):

    """
    As a libxslt extension function, return a string containing the XSLForms
    path to the 'context' node, using the special "this-name" variable to
    complete the path if an attribute reference is required (as indicated by
    'attribute_ref' being set to true). If 'multivalue_name' is set, produce a
    reference to a multivalued field using the given string as the attribute
    name.
    """

    pctxt = libxslt.xpathParserContext(_obj=context)
    context = pctxt.context()
    node = context.contextNode()
    transform_context = context.transformContext()
    name_var = transform_context.variableLookup("this-name", None)
    if multivalue_name is not None:
        name = multivalue_name
        multivalue = 1
    elif name_var is not None:
        name = name_var[0].content
        multivalue = 0
    else:
        name = None
        multivalue = 0
    return path_to_node(node, attribute_ref, name, multivalue)

def this_position(context):
    return path_to_context(context, 0)

def field_name(context):
    return path_to_context(context, 1)

def multi_field_name(context, multivalue_name):
    return path_to_context(context, 1, multivalue_name)

def new_field(context, name):
    return path_to_context(context, 0) + "/" + name

libxslt.registerExtModuleFunction("this-position", "http://www.boddie.org.uk/ns/xmltools/template", this_position)
libxslt.registerExtModuleFunction("field-name", "http://www.boddie.org.uk/ns/xmltools/template", field_name)
libxslt.registerExtModuleFunction("multi-field-name", "http://www.boddie.org.uk/ns/xmltools/template", multi_field_name)
libxslt.registerExtModuleFunction("new-field", "http://www.boddie.org.uk/ns/xmltools/template", new_field)

# vim: tabstop=4 expandtab shiftwidth=4
